##
##    SQLSDC -- A SessionDataContainer implemented over an Oracle RDBMS
##    Written By Shai Berger
##    Copyright (C) 2001 Aristocart, Inc.
##
##    This library is free software; you can redistribute it and/or
##    modify it under the terms of the GNU Library General Public
##    License as published by the Free Software Foundation; either
##    version 2 of the License, or (at your option) any later version.
##
##    This library is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##    Library General Public License for more details.
##
##    You should have received a copy of the GNU Library General Public
##    License along with this library; if not, write to the Free
##    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
##    MA 02111-1307, USA
##
## This file is intended for two missions:
## 1) Overcome Oracle's limitations on command sizes
## 2) Enable intelligent, retrying locking of database rows
##

import time

USE_DCORACLE2 = 0

if USE_DCORACLE2:
    # This assumes DCOracle2 is installed in Products/ZOracle2DA.
    # If this is not the case, change this line accordingly.
    from Products.ZOracle2DA.DCOracle2 import dbiRaw, Error
else:
    from Products.ZOracleDA.DCOracle import dbi, error
    dbiRaw = dbi.dbiRaw
    Error = error

# These, of course, depend on database definitions.
long_table_name = 'S_DATA'
long_column_name = 'S_OBJECT'
long_seq_name = 'SEQ_S_DATA'
index_column_name = 'S_DATA_ID'

save_command = '''
INSERT INTO %s (%s)
VALUES (:content)
'''                    % (long_table_name,long_column_name)

get_key_command = 'SELECT %s.currval FROM dual' % long_seq_name

#
# Put a long raw on the DB
#
def SaveLONG(self,i_connection, content):
    """Put a long raw on the DB"""
    content=str(content)
    db_content=dbiRaw(content)
    if content and content=='': return 0
    
    connection = i_connection()
    connection._register() # play nice with the transaction
    db = connection.cursor
    db.execute(save_command, content=db_content)
    db.execute(get_key_command)
    data_key=db.fetchone()[0]
    return data_key

load_command = '''
SELECT %s
FROM %s
WHERE %s=:key
'''             % (long_column_name, long_table_name, index_column_name)

#
# Get a long raw from the DB --- Not necessary, as ZSQL methods 
# can do this on their own.
#
def LoadLONG(self, connection, key):
    "Read a long raw from db"
    connection = connection()
    connection._register() # play nice with the transaction
    db = connection.cursor # the real DB connection object
    db.execute(load_command, key=key)
    rows = db.fetchmany(10)
    if not rows:
        return None
    else:
        return rows[0][0]


#
# Put a live Python object on the database
#
def SaveOBJ(self,connection, content):
    data = cPickle.dumps(content)
    return SaveLONG(self,connection,data)
#
# Restore a live Python object on the database
#
def LoadOBJ(self, connection, key):
    data = LoadLONG (self, connection, key)
    return cPickle.loads(data)

#
# Acquire a lock on an item
#
RESOURCE_BUSY=54                        # Oracle 8.0.5

def lock (self, key, timeout_mins, retries=10, wait=1):
    """Get an item locked. Relies on s_lock from environment"""
    class Locker:
        def __init__(self, func, key, timeout_mins):            
            self.func = func
            self.key = key
            self.timeout_mins = timeout_mins
        def __call__(self):
            return self.func(key=self.key,timeout_mins=self.timeout_mins)
            
    return _lock (Locker(self.s_lock, key, timeout_mins), retries, wait)
    
def lock_all(self, retries=10, wait=1):
    """Get a table locked. Relies on s_lock_all from environment"""
    return _lock (self.s_lock_all, retries, wait)


def _lock(locker, retries, wait):
    """Retry retries times, with wait seconds between them"""
    for retry in range(retries):
        try:
            return locker()
            break
        except Error, args:
            if args[0]==RESOURCE_BUSY and retry+1 < retries:
                time.sleep(wait)
            else:
                raise
        
